#ifndef COMPLEX_HH
#define COMPLEX_HH

#include <complex>

typedef std::complex< float >  fComplex;
typedef std::complex< double > dComplex;

// Functions that are not members of the complex class.

template < class T >
std::complex< T >
xcc( std::complex< T >& lhs, std::complex< T > rhs )
{
    lhs = lhs * conj( rhs );
    return lhs;
}

// Operators added to the complex class.

template < class T >
std::complex< T >
operator~( std::complex< T > z )
{
    return conj( z );
}

template < class T >
bool
operator!( std::complex< T > z )
{
    return ( !z.real( ) && !z.imag( ) );
}

inline std::complex< float >
operator*( float lhs, const std::complex< float >& rhs )
{
    std::complex< float > r( rhs );
    r *= lhs;
    return r;
}

inline std::complex< double >
operator*( float lhs, const std::complex< double >& rhs )
{
    std::complex< double > r( rhs );
    r *= lhs;
    return r;
}

inline std::complex< double >
operator*( const std::complex< float >& lhs, double rhs )
{
    std::complex< float > r = lhs;
    r *= rhs;
    return r;
}

inline std::complex< double >
operator*( const std::complex< double >& lhs, float rhs )
{
    std::complex< double > r = lhs;
    r *= rhs;
    return r;
}

inline std::complex< double >
operator/( const std::complex< double >& lhs, int rhs )
{
    std::complex< double > r( lhs );
    r /= rhs;
    return r;
}

inline std::complex< double >
operator+( const std::complex< float >& lhs, const std::complex< double >& rhs )
{
    std::complex< float > r = lhs;
    r += rhs;
    return r;
}

#endif
